@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Modified 2013-2016 Qball  Cow <qball@gmpclient.org>
+ * Modified 2013-2017 Qball  Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -33,10 +33,6 @@ Settings config = {
     /** List of enabled modi. */
     /** -modi */
     .modi              = "window,run,ssh",
-    /** Set the default window opacity. */
-    /** This option only works when running a composite manager. */
-    /** -o */
-    .window_opacity    =                                       100,
     /** Border width around the window. */
     .menu_bw           =                                         1,
     /** The width of the switcher. (0100 in % > 100 in pixels) */
@@ -86,8 +82,8 @@ Settings config = {
     .y_offset          =                                         0,
     /** X offset */
     .x_offset          =                                         0,
-    /** Always should config.menu_lines lines, even if less lines are available */
-    .fixed_num_lines   = FALSE,
+    /** Always show config.menu_lines lines, even if less lines are available */
+    .fixed_num_lines   = TRUE,
     /** Do not use history */
     .disable_history   = FALSE,
     /** Use levenshtein sorting when matching */
@@ -107,14 +103,15 @@ Settings config = {
     /** Parse ~/.ssh/known_hosts file in ssh view. */
     .parse_known_hosts = TRUE,
     /** Modi to combine into one view. */
-    .combi_modi = "window,run",
-    .glob       = FALSE,
-    .tokenize   = TRUE,
-    .regex      = FALSE,
+    .combi_modi      = "window,run",
+    .tokenize        = TRUE,
+    .matching        = "normal",
+    .matching_method = MM_NORMAL,
     /** Monitor */
-    .monitor           =                                        -1,
+    .monitor           = "-5",
     /** set line margin */
-    .line_margin       =                                         2,
+    .line_margin  =                                         2,
+    .line_padding =                                         1,
     /** Set filter */
     .filter            = NULL,
     /** Separator style: dash/solid */
@@ -124,9 +121,11 @@ Settings config = {
     .fullscreen        = FALSE,
     .fake_transparency = FALSE,
     .dpi               =                                        -1,
-    .threads           =                                         1,
+    .threads           =                                         0,
     .scrollbar_width   =                                         8,
     .scroll_method     =                                         0,
     .fake_background   = "screenshot",
     .window_format     = "{w}   {c}   {t}",
+    .click_to_exit     = TRUE,
+    .show_match        = TRUE,
 };
