@@ -1,6 +1,9 @@
 #ifndef ROFI_DIALOGS_DIALOGS_H
 #define ROFI_DIALOGS_DIALOGS_H
 
+/**
+ * @defgroup MODES Modes
+ */
 /**
  * List of available dialogs.
  */
@@ -12,4 +15,5 @@
 #include "dialogs/script.h"
 #include "dialogs/window.h"
 #include "dialogs/combi.h"
+#include "dialogs/help-keys.h"
 #endif // ROFI_DIALOGS_DIALOGS_H
