@@ -36,16 +36,25 @@ unsigned int rofi_get_num_enabled_modi ( void );
  */
 const Mode * rofi_get_mode ( unsigned int index );
 
+/**
+ * @param code the code to return
+ *
+ * Return value are used for integrating dmenu rofi in scripts.
+ * This function sets the code that rofi will return on exit.
+ */
 void rofi_set_return_code ( int code );
 /** Reset terminal */
-#define  color_reset        "\033[0m"
+#define  color_reset     "\033[0m"
 /** Set terminal text bold */
-#define  color_bold         "\033[1m"
+#define  color_bold      "\033[1m"
 /** Set terminal text italic */
-#define  color_italic       "\033[2m"
+#define  color_italic    "\033[2m"
 /** Set terminal foreground text green */
-#define color_green         "\033[0;32m"
+#define color_green      "\033[0;32m"
+/** Set terminal foreground text red */
+#define color_red        "\033[0;31m"
 
+/** Appends instructions on how to report an error. */
 #define ERROR_MSG( a )    a "\n"                                       \
     "If you suspect this is caused by a bug in rofi,\n"                \
     "please report the following information to rofi's github page:\n" \
@@ -54,6 +63,7 @@ void rofi_set_return_code ( int code );
     " * Steps to reproduce\n"                                          \
     " * The version of rofi you are running\n\n"                       \
     " <i>https://github.com/DaveDavenport/rofi/</i>"
+/** Indicates if ERROR_MSG uses pango markup */
 #define ERROR_MSG_MARKUP    TRUE
 /*@}*/
 #endif
