@@ -1,6 +1,21 @@
 #ifndef ROFI_SETTINGS_H
 #define ROFI_SETTINGS_H
 
+#include <glib.h>
+
+/**
+ * Enumeration indicating the matching method to use.
+ *
+ * @ingroup CONFIGURATION
+ */
+typedef enum
+{
+    MM_NORMAL = 0,
+    MM_REGEX  = 1,
+    MM_GLOB   = 2,
+    MM_FUZZY  = 3
+} MatchingMethod;
+
 /**
  * Enumeration indicating location or gravity of window.
  *
@@ -40,8 +55,6 @@ typedef struct
 {
     /** List of enabled modi */
     char           *modi;
-    /** Window settings */
-    unsigned int   window_opacity;
     /** Border width */
     unsigned int   menu_bw;
     /** Width (0-100 in %, > 100 in pixels, < 0 in char width.) */
@@ -105,13 +118,14 @@ typedef struct
     unsigned int   parse_known_hosts;
     /** Combi Modes */
     char           *combi_modi;
-    unsigned int   glob;
+    char           *matching;
+    MatchingMethod matching_method;
     unsigned int   tokenize;
-    unsigned int   regex;
     /** Monitors */
-    int            monitor;
+    char           *monitor;
     /** Line margin */
     unsigned int   line_margin;
+    unsigned int   line_padding;
     /** filter */
     char           *filter;
     /** style */
@@ -132,6 +146,9 @@ typedef struct
     char           *fake_background;
 
     char           *window_format;
+    /** Click outside the window to exit */
+    int            click_to_exit;
+    gboolean       show_match;
 } Settings;
 /** Global Settings structure. */
 extern Settings config;
