@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2016 Qball Cow <qball@gmpclient.org>
+ * Copyright 2013-2017 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -34,15 +34,20 @@
 #include <ctype.h>
 #include <stdint.h>
 #include <errno.h>
+#include <gio/gio.h>
+#include <gio/gunixinputstream.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include "rofi.h"
 #include "settings.h"
-#include "textbox.h"
+#include "widgets/textbox.h"
 #include "dialogs/dmenu.h"
 #include "helper.h"
 #include "xrmoptions.h"
 #include "view.h"
-// We limit at 1000000 rows for now.
-#define DMENU_MAX_ROWS    1000000
+
+#define LOG_DOMAIN    "Dialogs.DMenu"
 
 struct range_pair
 {
@@ -67,8 +72,6 @@ static inline void bittoggle ( uint32_t *array, unsigned int index )
 typedef struct
 {
     /** Settings */
-    // Prompt
-    char              *prompt;
     // Separator.
     char              separator;
 
@@ -84,6 +87,7 @@ typedef struct
     unsigned int      do_markup;
     // List with entries.
     char              **cmd_list;
+    unsigned int      cmd_list_real_length;
     unsigned int      cmd_list_length;
     unsigned int      only_selected;
     unsigned int      selected_count;
@@ -91,47 +95,95 @@ typedef struct
     gchar             **columns;
     gchar             *column_separator;
     gboolean          multi_select;
+
+    GCancellable      *cancel;
+    gulong            cancel_source;
+    GInputStream      *input_stream;
+    GDataInputStream  *data_input_stream;
 } DmenuModePrivateData;
 
-static char **get_dmenu ( DmenuModePrivateData *pd, FILE *fd, unsigned int *length )
+static void async_close_callback ( GObject *source_object, GAsyncResult *res, G_GNUC_UNUSED gpointer user_data )
 {
-    TICK_N ( "Read stdin START" );
-    char         **retv   = NULL;
-    unsigned int rvlength = 1;
+    g_input_stream_close_finish ( G_INPUT_STREAM ( source_object ), res, NULL );
+    g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Closing data stream." );
+}
 
-    *length = 0;
-    gchar   *data  = NULL;
-    size_t  data_l = 0;
-    ssize_t l      = 0;
-    while ( ( l = getdelim ( &data, &data_l, pd->separator, fd ) ) > 0 ) {
-        if ( rvlength < ( *length + 2 ) ) {
-            rvlength *= 2;
-            retv      = g_realloc ( retv, ( rvlength ) * sizeof ( char* ) );
+static void read_add ( DmenuModePrivateData * pd, char *data, gsize len )
+{
+    if ( ( pd->cmd_list_length + 2 ) > pd->cmd_list_real_length ) {
+        pd->cmd_list_real_length = MAX ( pd->cmd_list_real_length * 2, 512 );
+        pd->cmd_list             = g_realloc ( pd->cmd_list, ( pd->cmd_list_real_length ) * sizeof ( char* ) );
+    }
+    char *utfstr = rofi_force_utf8 ( data, len );
+
+    pd->cmd_list[pd->cmd_list_length]     = utfstr;
+    pd->cmd_list[pd->cmd_list_length + 1] = NULL;
+
+    pd->cmd_list_length++;
+}
+static void async_read_callback ( GObject *source_object, GAsyncResult *res, gpointer user_data )
+{
+    GDataInputStream     *stream = (GDataInputStream *) source_object;
+    DmenuModePrivateData *pd     = (DmenuModePrivateData *) user_data;
+    gsize                len;
+    char                 *data = g_data_input_stream_read_upto_finish ( stream, res, &len, NULL );
+    if ( data != NULL ) {
+        // Absorb separator, already in buffer so should not block.
+        g_data_input_stream_read_byte ( stream, NULL, NULL );
+        read_add ( pd, data, len );
+        g_free ( data );
+        rofi_view_reload ();
+
+        g_data_input_stream_read_upto_async ( pd->data_input_stream, &( pd->separator ), 1, G_PRIORITY_LOW, pd->cancel,
+                                              async_read_callback, pd );
+        return;
+    } else {
+        GError *error = NULL;
+        // Absorb separator, already in buffer so should not block.
+        // If error == NULL end of stream..
+        g_data_input_stream_read_byte ( stream, NULL, &error);
+        if (  error == NULL ) {
+            // Add empty line.
+            read_add ( pd, "", 0);
+            rofi_view_reload ();
+
+            g_data_input_stream_read_upto_async ( pd->data_input_stream, &( pd->separator ), 1, G_PRIORITY_LOW, pd->cancel,
+                    async_read_callback, pd );
+            return;
+        } else {
+            g_error_free (error);
         }
-        if ( data[l - 1] == pd->separator ) {
-            data[l - 1] = '\0';
-            l--;
-        }
-        char *utfstr = rofi_force_utf8 ( data, l );
+    }
+    if ( !g_cancellable_is_cancelled ( pd->cancel ) ) {
+        // Hack, don't use get active.
+        rofi_view_set_overlay ( rofi_view_get_active (), NULL );
+        g_input_stream_close_async ( G_INPUT_STREAM ( stream ), G_PRIORITY_LOW, pd->cancel, async_close_callback, pd );
+    }
+}
 
-        retv[( *length )] = utfstr;
+static void async_read_cancel ( G_GNUC_UNUSED GCancellable *cancel, G_GNUC_UNUSED gpointer data )
+{
+    g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Cancelled the async read." );
+}
 
-        ( *length )++;
-        // Stop when we hit 2³¹ entries.
-        if ( ( *length ) >= DMENU_MAX_ROWS ) {
+static void get_dmenu_async ( DmenuModePrivateData *pd )
+{
+    g_data_input_stream_read_upto_async ( pd->data_input_stream, &( pd->separator ), 1, G_PRIORITY_LOW, pd->cancel,
+                                          async_read_callback, pd );
+}
+static void get_dmenu_sync ( DmenuModePrivateData *pd )
+{
+    while  ( TRUE ) {
+        gsize len   = 0;
+        char  *data = g_data_input_stream_read_upto ( pd->data_input_stream, &( pd->separator ), 1, &len, NULL, NULL );
+        if ( data == NULL ) {
             break;
         }
+        g_data_input_stream_read_byte ( pd->data_input_stream, NULL, NULL );
+        read_add ( pd, data, len );
+        g_free ( data );
     }
-    if ( data != NULL ) {
-        free ( data );
-        data = NULL;
-    }
-    if ( retv != NULL ) {
-        retv               = g_realloc ( retv, ( *length + 1 ) * sizeof ( char* ) );
-        retv[( *length ) ] = NULL;
-    }
-    TICK_N ( "Read stdin STOP" );
-    return retv;
+    g_input_stream_close_async ( G_INPUT_STREAM ( pd->input_stream ), G_PRIORITY_LOW, pd->cancel, async_close_callback, pd );
 }
 
 static unsigned int dmenu_mode_get_num_entries ( const Mode *sw )
@@ -266,12 +318,16 @@ static void dmenu_output_formatted_line ( const char *format, const char *string
             g_free ( quote );
         }
         else if ( format[i] == 'f' ) {
-            fputs ( filter, stdout );
+            if ( filter ) {
+                fputs ( filter, stdout );
+            }
         }
         else if ( format[i] == 'F' ) {
-            char *quote = g_shell_quote ( filter );
-            fputs ( quote, stdout );
-            g_free ( quote );
+            if ( filter ) {
+                char *quote = g_shell_quote ( filter );
+                fputs ( quote, stdout );
+                g_free ( quote );
+            }
         }
         else {
             fputc ( format[i], stdout );
@@ -287,6 +343,21 @@ static void dmenu_mode_free ( Mode *sw )
     }
     DmenuModePrivateData *pd = (DmenuModePrivateData *) mode_get_private_data ( sw );
     if ( pd != NULL ) {
+        if ( pd->cancel  ) {
+            // If open, cancel reads.
+            if ( pd->input_stream && !g_input_stream_is_closed ( pd->input_stream ) ) {
+                g_cancellable_cancel ( pd->cancel );
+            }
+            // This blocks until cancel is done.
+            g_cancellable_disconnect ( pd->cancel, pd->cancel_source );
+            if ( pd->input_stream ) {
+                // Should close the stream if not yet done.
+                g_object_unref ( pd->data_input_stream );
+                g_object_unref ( pd->input_stream );
+            }
+            g_object_unref ( pd->cancel );
+        }
+
         for ( size_t i = 0; i < pd->cmd_list_length; i++ ) {
             if ( pd->cmd_list[i] ) {
                 free ( pd->cmd_list[i] );
@@ -310,7 +381,6 @@ static int dmenu_mode_init ( Mode *sw )
     mode_set_private_data ( sw, g_malloc0 ( sizeof ( DmenuModePrivateData ) ) );
     DmenuModePrivateData *pd = (DmenuModePrivateData *) mode_get_private_data ( sw );
 
-    pd->prompt        = "dmenu ";
     pd->separator     = '\n';
     pd->selected_line = UINT32_MAX;
 
@@ -319,8 +389,6 @@ static int dmenu_mode_init ( Mode *sw )
     // Input data separator.
     find_arg_char ( "-sep", &( pd->separator ) );
 
-    // Check prompt
-    find_arg_str (  "-p", &( pd->prompt ) );
     find_arg_uint (  "-selected-row", &( pd->selected_line ) );
     // By default we print the unescaped line back.
     pd->format = "s";
@@ -355,12 +423,12 @@ static int dmenu_mode_init ( Mode *sw )
     if ( find_arg ( "-i" ) >= 0 ) {
         config.case_sensitive = FALSE;
     }
-    FILE *fd = NULL;
+    int fd = STDIN_FILENO;
     str = NULL;
     if ( find_arg_str ( "-input", &str ) ) {
         char *estr = rofi_expand_path ( str );
-        fd = fopen ( str, "r" );
-        if ( fd == NULL ) {
+        fd = open ( str, O_RDONLY );
+        if ( fd < 0 ) {
             char *msg = g_markup_printf_escaped ( "Failed to open file: <b>%s</b>:\n\t<i>%s</i>", estr, strerror ( errno ) );
             rofi_view_error_dialog ( msg, TRUE );
             g_free ( msg );
@@ -369,10 +437,10 @@ static int dmenu_mode_init ( Mode *sw )
         }
         g_free ( estr );
     }
-    pd->cmd_list = get_dmenu ( pd, fd == NULL ? stdin : fd, &( pd->cmd_list_length ) );
-    if ( fd != NULL ) {
-        fclose ( fd );
-    }
+    pd->cancel            = g_cancellable_new ();
+    pd->cancel_source     = g_cancellable_connect ( pd->cancel, G_CALLBACK ( async_read_cancel ), pd, NULL );
+    pd->input_stream      = g_unix_input_stream_new ( fd, fd != STDIN_FILENO );
+    pd->data_input_stream = g_data_input_stream_new ( pd->input_stream );
 
     gchar *columns = NULL;
     if ( find_arg_str ( "-display-columns", &columns ) ) {
@@ -390,6 +458,7 @@ static int dmenu_token_match ( const Mode *sw, GRegex **tokens, unsigned int ind
 }
 
 #include "mode-private.h"
+/** dmenu Mode object. */
 Mode dmenu_mode =
 {
     .name               = "dmenu",
@@ -403,7 +472,8 @@ Mode dmenu_mode =
     ._get_completion    = NULL,
     ._preprocess_input  = NULL,
     .private_data       = NULL,
-    .free               = NULL
+    .free               = NULL,
+    .display_name       = "dmenu:"
 };
 
 static void dmenu_finish ( RofiViewState *state, int retv )
@@ -540,11 +610,24 @@ static void dmenu_finalize ( RofiViewState *state )
 int dmenu_switcher_dialog ( void )
 {
     mode_init ( &dmenu_mode );
-    MenuFlags            menu_flags      = MENU_NORMAL;
-    DmenuModePrivateData *pd             = (DmenuModePrivateData *) dmenu_mode.private_data;
-    char                 *input          = NULL;
-    unsigned int         cmd_list_length = pd->cmd_list_length;
-    char                 **cmd_list      = pd->cmd_list;
+    MenuFlags            menu_flags = MENU_NORMAL;
+    DmenuModePrivateData *pd        = (DmenuModePrivateData *) dmenu_mode.private_data;
+    int                  async      = TRUE;
+    // For now these only work in sync mode.
+    if ( find_arg ( "-sync" ) >= 0 || find_arg ( "-dump" ) >= 0 || find_arg ( "-select" ) >= 0
+         || find_arg ( "-no-custom" ) >= 0 || find_arg ( "-only-match" ) >= 0 || config.auto_select ||
+         find_arg ( "-selected-row" ) >= 0 ) {
+        async = FALSE;
+    }
+    if ( async ) {
+        get_dmenu_async ( pd );
+    }
+    else {
+        get_dmenu_sync ( pd );
+    }
+    char         *input          = NULL;
+    unsigned int cmd_list_length = pd->cmd_list_length;
+    char         **cmd_list      = pd->cmd_list;
 
     pd->only_selected = FALSE;
     pd->multi_select  = FALSE;
@@ -588,7 +671,7 @@ int dmenu_switcher_dialog ( void )
         GRegex       **tokens = tokenize ( config.filter ? config.filter : "", config.case_sensitive );
         unsigned int i        = 0;
         for ( i = 0; i < cmd_list_length; i++ ) {
-            if ( token_match ( tokens, cmd_list[i] ) ) {
+            if ( tokens == NULL || token_match ( tokens, cmd_list[i] ) ) {
                 dmenu_output_formatted_line ( pd->format, cmd_list[i], i, config.filter );
             }
         }
@@ -597,8 +680,12 @@ int dmenu_switcher_dialog ( void )
         g_free ( input );
         return TRUE;
     }
-    // TODO remove
-    RofiViewState *state = rofi_view_create ( &dmenu_mode, input, pd->prompt, pd->message, menu_flags, dmenu_finalize );
+    find_arg_str (  "-p", &( dmenu_mode.display_name ) );
+    RofiViewState *state = rofi_view_create ( &dmenu_mode, input, pd->message, menu_flags, dmenu_finalize );
+    // @TODO we should do this better.
+    if ( async ) {
+        rofi_view_set_overlay ( state, "Loading.. " );
+    }
     rofi_view_set_selected_line ( state, pd->selected_line );
     rofi_view_set_active ( state );
 
@@ -623,4 +710,5 @@ void print_dmenu_options ( void )
     print_help_msg ( "-markup-rows", "", "Allow and render pango markup as input data.", NULL, is_term );
     print_help_msg ( "-sep", "[char]", "Element separator.", "'\\n'", is_term );
     print_help_msg ( "-input", "[filename]", "Read input from file instead from standard input.", NULL, is_term );
+    print_help_msg ( "-sync", "", "Force dmenu to first read all input data, then show dialog.", NULL, is_term );
 }
