@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2016 Qball Cow <qball@gmpclient.org>
+ * Copyright 2013-2017 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -54,6 +54,9 @@
  */
 #define RUN_CACHE_FILE    "rofi-3.runcache"
 
+/** The log domain of this dialog. */
+#define LOG_DOMAIN        "Dialogs.Run"
+
 /**
  * The internal data structure holding the private data of the Run Mode.
  */
@@ -120,17 +123,19 @@ static void exec_cmd ( const char *cmd, int run_in_term )
         return;
     }
 
+    char *path = g_build_filename ( cache_dir, RUN_CACHE_FILE, NULL );
     if (  execsh ( lf_cmd, run_in_term ) ) {
         /**
          * This happens in non-critical time (After launching app)
          * It is allowed to be a bit slower.
          */
-        char *path = g_build_filename ( cache_dir, RUN_CACHE_FILE, NULL );
 
         history_set ( path, cmd );
-
-        g_free ( path );
     }
+    else {
+        history_remove ( path, cmd );
+    }
+    g_free ( path );
     g_free ( lf_cmd );
 }
 
@@ -249,23 +254,26 @@ static char ** get_apps ( unsigned int *length )
     gsize l        = 0;
     gchar *homedir = g_locale_to_utf8 (  g_get_home_dir (), -1, NULL, &l, &error );
     if ( error != NULL ) {
-        fprintf ( stderr, "Failed to convert homedir to UTF-8: %s\n", error->message );
+        g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Failed to convert homedir to UTF-8: %s", error->message );
         g_clear_error ( &error );
         g_free ( homedir );
         return NULL;
     }
 
-    const char *const sep = ":";
-    for ( const char *dirname = strtok ( path, sep ); dirname != NULL; dirname = strtok ( NULL, sep ) ) {
+    const char *const sep                 = ":";
+    char              *strtok_savepointer = NULL;
+    for ( const char *dirname = strtok_r ( path, sep, &strtok_savepointer ); dirname != NULL; dirname = strtok_r ( NULL, sep, &strtok_savepointer ) ) {
         DIR *dir = opendir ( dirname );
 
+        g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Checking path %s for executable.", dirname );
         if ( dir != NULL ) {
             struct dirent *dent;
             gsize         dirn_len = 0;
             gchar         *dirn    = g_locale_to_utf8 ( dirname, -1, NULL, &dirn_len, &error );
             if ( error != NULL ) {
-                fprintf ( stderr, "Failed to convert directory name to UTF-8: %s\n", error->message );
+                g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Failed to convert directory name to UTF-8: %s", error->message );
                 g_clear_error ( &error );
+                closedir ( dir );
                 continue;
             }
             gboolean is_homedir = g_str_has_prefix ( dirn, homedir );
@@ -291,7 +299,7 @@ static char ** get_apps ( unsigned int *length )
                 gsize name_len;
                 gchar *name = g_filename_to_utf8 ( dent->d_name, -1, NULL, &name_len, &error );
                 if ( error != NULL ) {
-                    fprintf ( stderr, "Failed to convert filename to UTF-8: %s\n", error->message );
+                    g_log ( LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Failed to convert filename to UTF-8: %s", error->message );
                     g_clear_error ( &error );
                     g_free ( name );
                     continue;
