@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright 2013-2016 Qball Cow <qball@gmpclient.org>
+ * Copyright 2013-2017 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -146,9 +146,7 @@ static ModeMode script_mode_result ( Mode *sw, int mretv, char **input, unsigned
 
         rmpd->cmd_list        = new_list;
         rmpd->cmd_list_length = new_length;
-        g_free ( *input );
-        *input = NULL;
-        retv   = RELOAD_DIALOG;
+        retv   = RESET_DIALOG;
     }
     return retv;
 }
@@ -209,4 +207,3 @@ Mode *script_switcher_parse_setup ( const char *str )
     script_switcher_free ( sw );
     return NULL;
 }
-
