@@ -3,7 +3,7 @@
  *
  * MIT/X11 License
  * Copyright (c) 2012 Sean Pringle <sean.pringle@gmail.com>
- * Modified 2013-2016 Qball Cow <qball@gmpclient.org>
+ * Modified 2013-2017 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -49,10 +49,17 @@
 #include "rofi.h"
 #include "view.h"
 
-extern xcb_connection_t *xcb_connection;
-extern xcb_screen_t     *xcb_screen;
-static int              stored_argc   = 0;
-static char             **stored_argv = NULL;
+const char *const monitor_position_entries[] = {
+    "on focused monitor",
+    "on focused window",
+    "at mouse pointer",
+    "on monitor with focused window",
+    "on monitor that has mouse pointer"
+};
+/** copy of the argc for use in commandline argument parser. */
+static int        stored_argc = 0;
+/** copy of the argv pointer for use in the commandline argument parser */
+static char       **stored_argv = NULL;
 
 void cmd_set_arguments ( int argc, char **argv )
 {
@@ -161,28 +168,70 @@ static gchar *glob_to_regex ( const char *input )
     }
     return r;
 }
+static gchar *fuzzy_to_regex ( const char * input )
+{
+    GString *str = g_string_new ( "" );
+    gchar   *r   = g_regex_escape_string ( input, -1 );
+    gchar   *iter;
+    int     first = 1;
+    for ( iter = r; iter && *iter != '\0'; iter = g_utf8_next_char ( iter ) ) {
+        if ( first ) {
+            g_string_append ( str, "(" );
+        }
+        else {
+            g_string_append ( str, ".*(" );
+        }
+        if ( *iter == '\\' ){
+            g_string_append_c ( str, '\\');
+            iter = g_utf8_next_char ( iter );
+            // If EOL, break out of for loop.
+            if ( (*iter) == '\0' ) break;
+        }
+        g_string_append_unichar ( str, g_utf8_get_char ( iter ) );
+        g_string_append ( str, ")" );
+        first = 0;
+    }
+    g_free ( r );
+    char *retv = str->str;
+    g_string_free ( str, FALSE );
+    return retv;
+}
+
+// Macro for quickly generating regex for matching.
+static inline GRegex * R ( const char *s, int case_sensitive  )
+{
+    return g_regex_new ( s, G_REGEX_OPTIMIZE | ( ( case_sensitive ) ? 0 : G_REGEX_CASELESS ), 0, NULL );
+}
+
 static GRegex * create_regex ( const char *input, int case_sensitive )
 {
-#define R( s )    g_regex_new ( s, G_REGEX_OPTIMIZE | ( ( case_sensitive ) ? 0 : G_REGEX_CASELESS ), 0, NULL )
     GRegex * retv = NULL;
-    if ( config.glob ) {
-        gchar *r = glob_to_regex ( input );
-        retv = R ( r );
+    gchar  *r;
+    switch ( config.matching_method )
+    {
+    case MM_GLOB:
+        r    = glob_to_regex ( input );
+        retv = R ( r, case_sensitive );
         g_free ( r );
-    }
-    else if ( config.regex ) {
-        retv = R ( input );
+        break;
+    case MM_REGEX:
+        retv = R ( input, case_sensitive );
         if ( retv == NULL ) {
-            gchar *r = g_regex_escape_string ( input, -1 );
-            retv = R ( r );
+            r    = g_regex_escape_string ( input, -1 );
+            retv = R ( r, case_sensitive );
             g_free ( r );
         }
-    }
-    else{
-        // TODO; regex should be default?
-        gchar *r = g_regex_escape_string ( input, -1 );
-        retv = R ( r );
+        break;
+    case MM_FUZZY:
+        r    = fuzzy_to_regex ( input );
+        retv = R ( r, case_sensitive );
         g_free ( r );
+        break;
+    default:
+        r    = g_regex_escape_string ( input, -1 );
+        retv = R ( r, case_sensitive );
+        g_free ( r );
+        break;
     }
     return retv;
 }
@@ -269,47 +318,35 @@ int find_arg_uint ( const char * const key, unsigned int *val )
 
 char helper_parse_char ( const char *arg )
 {
-    int len = strlen ( arg );
+    const size_t len = strlen ( arg );
     // If the length is 1, it is not escaped.
     if ( len == 1 ) {
         return arg[0];
     }
     // If the length is 2 and the first character is '\', we unescape it.
     if ( len == 2 && arg[0] == '\\' ) {
+        switch ( arg[1] )
+        {
         // New line
-        if ( arg[1] == 'n' ) {
-            return '\n';
-        }
+        case 'n': return '\n';
         // Bell
-        else if ( arg[1] == 'a' ) {
-            return '\a';
-        }
+        case  'a': return '\a';
         // Backspace
-        else if ( arg[1] == 'b' ) {
-            return '\b';
-        }
+        case 'b': return '\b';
         // Tab
-        else if ( arg[1] == 't' ) {
-            return '\t';
-        }
+        case  't': return '\t';
         // Vertical tab
-        else if ( arg[1] == 'v' ) {
-            return '\v';
-        }
+        case  'v': return '\v';
         // Form feed
-        else if ( arg[1] == 'f' ) {
-            return '\f';
-        }
+        case  'f': return '\f';
         // Carriage return
-        else if ( arg[1] == 'r' ) {
-            return '\r';
-        }
+        case  'r': return '\r';
         // Forward slash
-        else if ( arg[1] == '\\' ) {
-            return '\\';
-        }
-        else if ( arg[1] == '0' ) {
-            return '\0';
+        case  '\\': return '\\';
+        // 0 line.
+        case  '0': return '\0';
+        default:
+            break;
         }
     }
     if ( len > 2 && arg[0] == '\\' && arg[1] == 'x' ) {
@@ -339,14 +376,17 @@ PangoAttrList *token_match_get_pango_attr ( GRegex **tokens, const char *input,
             GMatchInfo *gmi = NULL;
             g_regex_match ( (GRegex *) tokens[j], input, G_REGEX_MATCH_PARTIAL, &gmi );
             while ( g_match_info_matches ( gmi ) ) {
-                int            start, end;
-                g_match_info_fetch_pos ( gmi, 0, &start, &end );
-                PangoAttribute *pa  = pango_attr_underline_new ( PANGO_UNDERLINE_SINGLE );
-                PangoAttribute *pa2 = pango_attr_weight_new ( PANGO_WEIGHT_BOLD );
-                pa2->start_index = pa->start_index = start;
-                pa2->end_index   = pa->end_index = end;
-                pango_attr_list_insert ( retv, pa );
-                pango_attr_list_insert ( retv, pa2 );
+                int count = g_match_info_get_match_count ( gmi );
+                for ( int index = ( count > 1 ) ? 1 : 0; index < count; index++ ) {
+                    int            start, end;
+                    g_match_info_fetch_pos ( gmi, index, &start, &end );
+                    PangoAttribute *pa  = pango_attr_underline_new ( PANGO_UNDERLINE_SINGLE );
+                    PangoAttribute *pa2 = pango_attr_weight_new ( PANGO_WEIGHT_BOLD );
+                    pa2->start_index = pa->start_index = start;
+                    pa2->end_index   = pa->end_index = end;
+                    pango_attr_list_insert ( retv, pa );
+                    pango_attr_list_insert ( retv, pa2 );
+                }
                 g_match_info_next ( gmi, NULL );
             }
             g_match_info_free ( gmi );
@@ -355,14 +395,12 @@ PangoAttrList *token_match_get_pango_attr ( GRegex **tokens, const char *input,
     return retv;
 }
 
-int token_match ( GRegex **tokens, const char *input )
+int token_match ( GRegex * const *tokens, const char *input )
 {
-    int match = 1;
+    int match = ( tokens != NULL );
     // Do a tokenized match.
-    if ( tokens ) {
-        for ( int j = 0; match && tokens[j]; j++ ) {
-            match = g_regex_match ( (GRegex *) tokens[j], input, 0, NULL );
-        }
+    for ( int j = 0; match && tokens[j]; j++ ) {
+        match = g_regex_match ( (const GRegex *) tokens[j], input, 0, NULL );
     }
     return match;
 }
@@ -399,14 +437,14 @@ int create_pid_file ( const char *pidfile )
 
     int fd = g_open ( pidfile, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR );
     if ( fd < 0 ) {
-        fprintf ( stderr, "Failed to create pid file." );
+        fprintf ( stderr, "Failed to create pid file: '%s'.\n", pidfile );
         return -1;
     }
     // Set it to close the File Descriptor on exit.
     int flags = fcntl ( fd, F_GETFD, NULL );
     flags = flags | FD_CLOEXEC;
     if ( fcntl ( fd, F_SETFD, flags, NULL ) < 0 ) {
-        fprintf ( stderr, "Failed to set CLOEXEC on pidfile." );
+        fprintf ( stderr, "Failed to set CLOEXEC on pidfile.\n" );
         remove_pid_file ( fd );
         return -1;
     }
@@ -450,6 +488,27 @@ int config_sanity_check ( void )
     int     found_error = FALSE;
     GString *msg        = g_string_new (
         "<big><b>The configuration failed to validate:</b></big>\n" );
+
+    if ( config.matching ) {
+        if ( g_strcmp0 ( config.matching, "regex" ) == 0 ) {
+            config.matching_method = MM_REGEX;
+        }
+        else if ( g_strcmp0 ( config.matching, "glob" ) == 0 ) {
+            config.matching_method = MM_GLOB;
+        }
+        else if ( g_strcmp0 ( config.matching, "fuzzy" ) == 0 ) {
+            config.matching_method = MM_FUZZY;
+        }
+        else if ( g_strcmp0 ( config.matching, "normal" ) == 0 ) {
+            config.matching_method = MM_NORMAL;;
+        }
+        else {
+            g_string_append_printf ( msg, "\t<b>config.matching</b>=%s is not a valid matching strategy.\nValid options are: glob, regex, fuzzy or normal.\n",
+                                     config.matching );
+            found_error = 1;
+        }
+    }
+
     if ( config.element_height < 1 ) {
         g_string_append_printf ( msg, "\t<b>config.element_height</b>=%d is invalid. An element needs to be atleast 1 line high.\n",
                                  config.element_height );
@@ -476,25 +535,17 @@ int config_sanity_check ( void )
 
     // Check size
     {
-        int ssize = monitor_get_smallest_size ( );
-        if ( config.monitor >= 0 ) {
-            workarea mon;
-            if ( monitor_get_dimension ( config.monitor, &mon ) ) {
-                ssize = MIN ( mon.w, mon.h );
+        workarea mon;
+        if ( !monitor_active ( &mon ) ) {
+            const char *name = config.monitor;
+            if ( name && name[0] == '-' ) {
+                int index = name[1] - '0';
+                if ( index < 5 && index > 0 ) {
+                    name = monitor_position_entries[index - 1];
+                }
             }
-            else{
-                g_string_append_printf ( msg, "\t<b>config.monitor</b>=%d Could not find monitor.\n", config.monitor );
-                ssize = 0;
-            }
-        }
-        // Have todo an estimate here.
-        if ( ( 2 * ( config.padding + config.menu_bw ) ) > ( 0.9 * ssize ) ) {
-            g_string_append_printf ( msg, "\t<b>config.padding+config.menu_bw</b>=%d is too big for the minimum size of the monitor: %d.\n",
-                                     ( config.padding + config.menu_bw ), ssize );
-
-            config.padding = 0;
-            config.menu_bw = 0;
-            found_error    = TRUE;
+            g_string_append_printf ( msg, "\t<b>config.monitor</b>=%s Could not find monitor.\n", name );
+            found_error = TRUE;
         }
     }
 
@@ -511,7 +562,7 @@ int config_sanity_check ( void )
         pango_font_description_free ( pfd );
     }
 
-    if ( config.monitor == -3 ) {
+    if ( g_strcmp0 ( config.monitor, "-3" ) == 0 ) {
         // On -3, set to location 1.
         config.location   = 1;
         config.fullscreen = 0;
@@ -557,30 +608,30 @@ char *rofi_expand_path ( const char *input )
     return retv;
 }
 
+/** Return the minimum value of a,b,c */
 #define MIN3( a, b, c )    ( ( a ) < ( b ) ? ( ( a ) < ( c ) ? ( a ) : ( c ) ) : ( ( b ) < ( c ) ? ( b ) : ( c ) ) )
 
 unsigned int levenshtein ( const char *needle, const char *haystack )
 {
-    unsigned int x, y, lastdiag, olddiag;
-    size_t       needlelen   = g_utf8_strlen ( needle, -1 );
-    size_t       haystacklen = g_utf8_strlen ( haystack, -1 );
+    const size_t needlelen   = g_utf8_strlen ( needle, -1 );
+    const size_t haystacklen = g_utf8_strlen ( haystack, -1 );
     unsigned int column[needlelen + 1];
-    for ( y = 0; y <= needlelen; y++ ) {
+    for ( unsigned int y = 0; y <= needlelen; y++ ) {
         column[y] = y;
     }
-    for ( x = 1; x <= haystacklen; x++ ) {
+    for ( unsigned int x = 1; x <= haystacklen; x++ ) {
         const char *needles = needle;
         column[0] = x;
         gunichar   haystackc = g_utf8_get_char ( haystack );
         if ( !config.case_sensitive ) {
             haystackc = g_unichar_tolower ( haystackc );
         }
-        for ( y = 1, lastdiag = x - 1; y <= needlelen; y++ ) {
+        for ( unsigned int y = 1, lastdiag = x - 1; y <= needlelen; y++ ) {
             gunichar needlec = g_utf8_get_char ( needles );
             if ( !config.case_sensitive ) {
                 needlec = g_unichar_tolower ( needlec );
             }
-            olddiag   = column[y];
+            unsigned int olddiag = column[y];
             column[y] = MIN3 ( column[y] + 1, column[y - 1] + 1, lastdiag + ( needlec == haystackc ? 0 : 1 ) );
             lastdiag  = olddiag;
             needles   = g_utf8_next_char ( needles );
