@@ -33,7 +33,6 @@ static void history_test ( void )
 
     TASSERT ( retv != NULL );
     TASSERT ( length == 1 );
-
     TASSERT ( strcmp ( retv[0], "aap" ) == 0 );
 
     g_strfreev ( retv );
@@ -79,8 +78,16 @@ static void history_test ( void )
 
     TASSERT ( retv != NULL );
     TASSERT ( length == 25 );
+    for ( unsigned int in = 0; in < 24; in++ ) {
+        char *p = g_strdup_printf ( "aap%i", in + 2 );
+        TASSERT ( g_strcmp0 ( retv[in], p ) == 0 );
+
+        g_free ( p );
+    }
 
     g_strfreev ( retv );
+
+    unlink ( file );
 }
 
 int main (  G_GNUC_UNUSED int argc, G_GNUC_UNUSED char **argv )
